@@ -23,10 +23,11 @@
 */
 
 #include "konqfeedicon.h"
-#include "feeddetector.h"
-#include "pluginbase.h"
 
-#include <kdebug.h>
+#include "feeddetector.h"
+#include "pluginutil.h"
+#include "akregatorplugindebug.h"
+
 #include <kpluginfactory.h>
 #include <KLocalizedString>
 #include <kiconloader.h>
@@ -35,30 +36,19 @@
 #include <KParts/ReadOnlyPart>
 #include <KParts/HtmlExtension>
 #include <KParts/SelectorInterface>
-#include <kprocess.h>
-#include <kstandarddirs.h>
-#include <kstatusbar.h>
+#include <kio/job.h>
 #include <kurllabel.h>
-
 #include <kprotocolinfo.h>
 
-#include <qcursor.h>
-#include <qobject.h>
-#include <qpixmap.h>
-#include <qstringlist.h>
-#include <QTextDocument>
-#include <KLocalizedString>
-
-//KDELibs4Support
-#include <kurl.h>
+#include <qstatusbar.h>
 
 using namespace Akregator;
 
 K_PLUGIN_FACTORY(KonqFeedIconFactory, registerPlugin<KonqFeedIcon>();)
 
-static KUrl baseUrl(KParts::ReadOnlyPart *part)
+static QUrl baseUrl(KParts::ReadOnlyPart *part)
 {
-    KUrl url;
+    QUrl url;
     KParts::HtmlExtension *ext = KParts::HtmlExtension::childObject(part);
     if (ext) {
         url = ext->baseUrl();
@@ -66,9 +56,15 @@ static KUrl baseUrl(KParts::ReadOnlyPart *part)
     return url;
 }
 
-KonqFeedIcon::KonqFeedIcon(QObject *parent, const QVariantList &)
-    : KParts::Plugin(parent), PluginBase(), m_part(0), m_feedIcon(0), m_statusBarEx(0), m_menu(0)
+KonqFeedIcon::KonqFeedIcon(QObject *parent, const QVariantList &args)
+    : KParts::Plugin(parent),
+      m_part(nullptr),
+      m_feedIcon(nullptr),
+      m_statusBarEx(nullptr),
+      m_menu(nullptr)
 {
+    Q_UNUSED(args);
+
     // make our icon foundable by the KIconLoader
     KIconLoader::global()->addAppDir(QStringLiteral("akregator"));
 
@@ -78,9 +74,9 @@ KonqFeedIcon::KonqFeedIcon(QObject *parent, const QVariantList &)
         KParts::SelectorInterface *selectorInterface = qobject_cast<KParts::SelectorInterface *>(ext);
         if (selectorInterface) {
             m_part = part;
-            connect(m_part, SIGNAL(completed()), this, SLOT(addFeedIcon()));
-            connect(m_part, SIGNAL(completed(bool)), this, SLOT(addFeedIcon()));
-            connect(m_part, SIGNAL(started(KIO::Job*)), this, SLOT(removeFeedIcon()));
+            connect(m_part, QOverload<>::of(&KParts::ReadOnlyPart::completed), this, &KonqFeedIcon::addFeedIcon);
+            connect(m_part, QOverload<bool>::of(&KParts::ReadOnlyPart::completed), this, &KonqFeedIcon::addFeedIcon);
+            connect(m_part, &KParts::ReadOnlyPart::started, this, &KonqFeedIcon::removeFeedIcon);
         }
     }
 }
@@ -101,9 +97,14 @@ KonqFeedIcon::~KonqFeedIcon()
 
 bool KonqFeedIcon::feedFound()
 {
+    // Ensure that it is safe to use the URL, before doing anything else with it
+    const QUrl partUrl(m_part->url());
+    if (!partUrl.isValid()) {
+        return false;
+    }
     // Since attempting to determine feed info for about:blank crashes khtml,
     // lets prevent such look up for local urls (about, file, man, etc...)
-    if (KProtocolInfo::protocolClass(m_part->url().scheme()).compare(QLatin1String(":local"), Qt::CaseInsensitive) == 0) {
+    if (KProtocolInfo::protocolClass(partUrl.scheme()).compare(QLatin1String(":local"), Qt::CaseInsensitive) == 0) {
         return false;
     }
 
@@ -112,7 +113,6 @@ bool KonqFeedIcon::feedFound()
     QString doc;
     if (selectorInterface) {
         QList<KParts::SelectorInterface::Element> linkNodes = selectorInterface->querySelectorAll(QStringLiteral("head > link[rel=\"alternate\"]"), KParts::SelectorInterface::EntireContent);
-        //kDebug() << linkNodes.length() << "links";
         for (int i = 0; i < linkNodes.count(); i++) {
             const KParts::SelectorInterface::Element element = linkNodes.at(i);
 
@@ -126,7 +126,7 @@ bool KonqFeedIcon::feedFound()
             }
             doc += QLatin1String("/>");
         }
-        kDebug() << doc;
+        qCDebug(AKREGATORPLUGIN_LOG) << doc;
     }
 
     m_feedList = FeedDetector::extractFromLinkTags(doc);
@@ -139,7 +139,7 @@ void KonqFeedIcon::contextMenu()
     m_menu = new QMenu(m_part->widget());
     if (m_feedList.count() == 1) {
         m_menu->setTitle(m_feedList.first().title());
-        m_menu->addAction(SmallIcon("bookmark-new"), i18n("Add Feed to Akregator"), this, SLOT(addFeeds()));
+        m_menu->addAction(SmallIcon("bookmark-new"), i18n("Add Feed to Akregator"), this, SLOT(addAllFeeds()));
     } else {
         m_menu->setTitle(i18n("Add Feeds to Akregator"));
         int id = 0;
@@ -150,7 +150,7 @@ void KonqFeedIcon::contextMenu()
         }
         //disconnect(m_menu, SIGNAL(activated(int)), this, SLOT(addFeed(int)));
         m_menu->addSeparator();
-        m_menu->addAction(QIcon::fromTheme(QStringLiteral("bookmark-new")), i18n("Add All Found Feeds to Akregator"), this, SLOT(addFeeds()));
+        m_menu->addAction(QIcon::fromTheme(QStringLiteral("bookmark-new")), i18n("Add All Found Feeds to Akregator"), this, SLOT(addAllFeeds()));
     }
     m_menu->popup(QCursor::pos());
 }
@@ -174,12 +174,11 @@ void KonqFeedIcon::addFeedIcon()
     m_feedIcon->setUseCursor(false);
 
     m_feedIcon->setPixmap(KIconLoader::global()->loadIcon(QStringLiteral("feed"), KIconLoader::User));
-
     m_feedIcon->setToolTip(i18n("Subscribe to site updates (using news feed)"));
 
     m_statusBarEx->addStatusBarItem(m_feedIcon, 0, true);
 
-    connect(m_feedIcon, SIGNAL(leftClickedUrl()), this, SLOT(contextMenu()));
+    connect(m_feedIcon, QOverload<>::of(&KUrlLabel::leftClickedUrl), this, &KonqFeedIcon::contextMenu);
 }
 
 void KonqFeedIcon::removeFeedIcon()
@@ -203,34 +202,17 @@ void KonqFeedIcon::addFeed()
     if (!ok || id == -1) {
         return;
     }
-    if (akregatorRunning()) {
-        addFeedsViaDBUS(QStringList(fixRelativeURL(m_feedList[id].url(), baseUrl(m_part))));
-    } else {
-        addFeedViaCmdLine(fixRelativeURL(m_feedList[id].url(), baseUrl(m_part)));
-    }
+    PluginUtil::addFeeds(QStringList(PluginUtil::fixRelativeURL(m_feedList[id].url(), baseUrl(m_part))));
 }
 
 // from akregatorplugin.cpp
-void KonqFeedIcon::addFeeds()
+void KonqFeedIcon::addAllFeeds()
 {
-    if (akregatorRunning()) {
-        QStringList list;
-        for (FeedDetectorEntryList::Iterator it = m_feedList.begin(); it != m_feedList.end(); ++it) {
-            list.append(fixRelativeURL((*it).url(), baseUrl(m_part)));
-        }
-        addFeedsViaDBUS(list);
-    } else {
-        kDebug() << "KonqFeedIcon::addFeeds(): use command line";
-        KProcess proc;
-        proc << QStringLiteral("akregator") << QStringLiteral("-g") << i18n("Imported Feeds");
-
-        for (FeedDetectorEntryList::Iterator it = m_feedList.begin(); it != m_feedList.end(); ++it) {
-            proc << QStringLiteral("-a") << fixRelativeURL((*it).url(), baseUrl(m_part));
-        }
-
-        proc.startDetached();
-
+    QStringList list;
+    foreach (const FeedDetectorEntry &it, m_feedList) {
+        list.append(PluginUtil::fixRelativeURL(it.url(), baseUrl(m_part)));
     }
+    PluginUtil::addFeeds(list);
 }
 
 #include "konqfeedicon.moc"
